module Funkce where

import Database.HSQL.MySQL
import IO
import Char
import Random

-- Funkce (obslužné funkce pro databázi a menu programu H + M slovíčka)

-- Existuje tabulka?

tableExists :: Connection -> String -> IO Bool
tableExists db name = tables db >>= return . elem name

-- Vytvoření tabulky se slovíčky

tableCreate :: Connection -> String -> IO ()
tableCreate db name = do
	execute db ("create table " ++ name ++ "(id int auto_increment, primary key (id), cestina varchar(255) not null, unique (cestina), necestina varchar(255) not null, unique (necestina), x int not null default 0) default charset = 'latin2' collate = 'latin2_czech_cs'")

-- Zahození tabulky x (pouze pro testovací účely)

tableDrop :: Connection -> String -> IO ()
tableDrop db name = do
	execute db ("drop table " ++ name)

-- Zapiš nové slovíčko do databáze

insertData :: Connection -> String -> [String] -> IO ()
insertData db name v = do
	execute db ("insert into " ++ name ++ "(cestina, necestina, x) values ('" ++ escape (v !! 0) ++ "', '" ++ escape (v !! 1) ++ "', " ++ (v !! 2) ++ ")")

-- Nastav všechna hodnoty x na defaultní hodnoty

alignData :: Connection -> String -> IO ()
alignData db name = do
	execute db ("update " ++ name ++ " set x = id")

-- Zvyš x o jednu

addX :: Connection -> String -> String -> Char -> IO ()
addX db name x y = do
	if y == '1' then
		execute db ("update " ++ name ++ " set x = x + 1 where necestina = '" ++ escape (x) ++ "'")
		else
		execute db ("update " ++ name ++ " set x = x + 1 where cestina = '" ++ escape (x) ++ "'")

-- Sniž x jednu, pokud je větší než 1

subX :: Connection -> String -> String -> Char -> IO ()
subX db name x y = do
	if y == '1' then
		execute db ("update " ++ name ++ " set x = x - 1 where necestina = '" ++ escape (x) ++ "' and x > 1")
		else
		execute db ("update " ++ name ++ " set x = x - 1 where cestina = '" ++ escape (x) ++ "' and x > 1")

-- Vybere z tabulky všechny řádky (sloupec id)

retrieveCount :: Connection -> String -> IO [(Int)]
retrieveCount db name = do
	query db ("select id from " ++ name) >>= collectRows getRow
	where
		getRow :: Statement -> IO (Int)
		getRow stmt = do
			x <- getFieldValue' stmt "id" 0
			return (x)

-- Zjisti, jestli už položky v databázi nejsou a vrať id s tím řádkem

searchDuplicate :: Connection -> String -> String -> String -> IO [(Int)]
searchDuplicate db name x y = do
	query db ("select id from " ++ name ++ " where cestina like '" ++ escape (trim y) ++ "' or necestina like '" ++ escape (trim x) ++ "'") >>= collectRows getRow
	where
		getRow :: Statement -> IO (Int)
		getRow stmt = do
			x <- getFieldValue stmt "id"
			return (x)

-- Náhodný výběr záznamu z tabulky s omezením podle x a limitem 1

retrieveRecords :: Connection -> String -> Int -> IO [(String, String)]
retrieveRecords db name x = do
	query db ("select cestina, necestina from " ++ name ++ " where x >= " ++ [intToDigit x] ++ " order by rand() limit 1") >>= collectRows getRow
	where
		getRow :: Statement -> IO (String, String)
		getRow stmt = do
			cestina <- getFieldValue stmt "cestina"
			necestina <- getFieldValue stmt "necestina"
			return (cestina, necestina)

-- Vybere z tabulky největší hodnotu sloupce

selectBiggestX :: Connection -> String -> IO [(Int)]
selectBiggestX db name = do
	query db ("select x from " ++ name ++ " order by x desc limit 1") >>= collectRows getRow
	where
		getRow :: Statement -> IO (Int)
		getRow stmt = do
			x <- getFieldValue' stmt "x" 0
			return (x)

-- Zahodí všechny bílé znaky ze začátku a konce řetězce

trim :: String -> String
trim = applyTwice (reverse . trim1)
	where
		trim1 = dropWhile (`elem` delim)
		delim = [' ', '\t', '\n', '\r']
		applyTwice f = f . f

-- Přidá znak "\" před znak "'" (escape sekvence, ochrana před SQL injekcí)

escape :: String -> String
escape [] = []
escape (x:s) = if x == '\'' then
		'\\' : x : escape s
		else
		x : escape s

-- Hlavní menu programu

menu :: Connection -> String -> Char -> IO ()
menu db name x = do
	if x == '3' then
		putStrLn "\nDěkuji, že používáte můj program."
		else if x == '1' then do
		cnt <- retrieveCount db name
		let count = (length cnt)
		if count == 0 then do
			putStrLn "\n\nNenalezena žádná slovíčka; není co procvičovat."
			menu db name '0'
			else do
			putStrLn ""
			putStrLn "[1] Čeština -> Cizí jazyk"
			putStrLn "[2] Cizí jazyk -> Čeština"
			putStrLn "[3] Zpět"
			putStr "Stiskněte klávesu z možností [1, 2, 3]: "
			getChar >>= exercise db name
		else if x == '2' then do
		putStr "\nNapište slovo v cizím jazyce: "
		slovo <- getLine
		putStr "Napište slovo v češtině: "
		getLine >>= insertion db name slovo
		else do
		putStrLn ""
		putStrLn "[1] Procvičit"
		putStrLn "[2] Vložit"
		putStrLn "[3] Ukončit"
		putStr "Stiskněte klávesu z možností [1, 2, 3]: "
		getChar >>= menu db name

-- Menu procvičování slovíček

exercise :: Connection -> String -> Char -> IO ()
exercise db name x = do
	biggest <- selectBiggestX db name
	rand <- randomRIO (1, (biggest !! 0))
	r <- retrieveRecords db name rand
	let ceske = fst (r !! 0)
	let neceske = snd (r !! 0)
	if x == '1' then do
		putStr ("\nPřeložte slovo '" ++ ceske ++ "' z češtiny: ")
		getLine >>= evaluate db name '1' neceske
		else if x == '2' then do
		putStr ("\nPřeložte slovo '" ++ neceske ++ "' do češtiny: ")
		getLine >>= evaluate db name '2' ceske
		else if x == '3' then
		menu db name '0'
		else
		menu db name '1'

-- Vyhodnocení opakování procvičování

exerciseLoop :: Connection -> String -> Char -> Char -> IO ()
exerciseLoop db name y x = do
	if x == 'a' || x == 'A' then
		if y == '1' then
			exercise db name '1'
			else
			exercise db name '2'
		else do
		alignData db name
		menu db name '0'

-- Vyhodnocení správnosti zadaného slovíčka

evaluate :: Connection -> String -> Char -> String -> String -> IO ()
evaluate db name z x y = do
	if trim (map toLower x) == trim (map toLower y) then do
		subX db name x z
		putStrLn ("Správně! :-) Odpověd zní '" ++ x ++ "'.")
		else do
		addX db name x z
		putStrLn ("Špatně. :-( Správná odpověď je '" ++ x ++ "'.")

	putStr "Další slovíčko? [A/N] "
	getChar >>= exerciseLoop db name z

-- Vložení slovíčka x a jeho překladu y do databáze

insertion :: Connection -> String -> String -> String -> IO ()
insertion db name x y = do
	if length (trim x) == 0 || length (trim y) == 0 then
		putStrLn "\nNezadali jste obě položky.\n"
		else do
		d <- searchDuplicate db name x y
		if length d > 0 then
			putStrLn "\nZadaná položka už existuje.\n"
			else do
			z <- selectBiggestX db name
			if length z == 0 then
				insertData db name ([(trim y), (trim x), "1"])
				else
				insertData db name ([(trim y), (trim x), [intToDigit ((z !! 0) + 1)]])
	putStr "Vložit další? [A/N] "
	getChar >>= insertionLoop db name

-- Vyhodnocení opakování vkládání slovíček

insertionLoop :: Connection -> String -> Char -> IO ()
insertionLoop db name x = do
	if x == 'a' || x == 'A' then
		menu db name '2'
		else
		menu db name '0'